local Class = require("lib.Class")
local Events = require("lib.Events")
local GPM = Class:derive("GamepadMgr")
local DEAD_ZONE = 0.1

local function hookLoveEvents(self)
   
   function love.joystickadded(joystick)
      local id = joystick:getID()
      assert(self.connectedSticks[id] == nil, "HATA: Joystick " .. id .. " already exists!")
      self.connectedSticks[id] = joystick
      self.isConnected[id] = true
      print("connected: " .. id)
      self.buttonMap[id] = {}
      self.event:add('controllerAdded', id)
      -- TODO: Invoke a 'controllerAdded' event
   end
   
   function love.joystickremoved(joystick)
      local id = joystick:getID()
      self.isConnected[id] = false
      self.connectedSticks[id] = nil
      self.buttonMap[id] = {}
      -- TODO: Invoke a 'controllerRemoved' event
      self.event:invoke('controllerRemoved', id)
   end
   
   function love.gamepadpressed(joystick, button)
      local id = joystick:getID()
      self.buttonMap[id][button] = true 
   end
   
   function love.gamepadreleased(joystick, button)
      local id = joystick:getID()
      self.buttonMap[id][button] = false 
   end
   
end

function GPM:new(dbFiles, adEnabled)
   if dbFiles ~= nil then
      for i = 1, #dbFiles do
	 love.joystick.loadGamepadMappings(dbFiles[i])
      end
   end

   self.event = Events()
   self.event:add('controllerAdded')
   self.event:add('controllerRemoved')
   
   hookLoveEvents(self)
   
   -- If it's true, then the left analog joystick will be converted to
   -- its corresponding dpad button output
   self.adEnabled = adEnabled
   
   -- The currently connected joysticks
   self.connectedSticks = {}
   self.isConnected = {}

   -- Maps a joystick id to a table of key values
   -- where the key is button and the value is either true = justPressed
   -- false = justReleased, nil = none
   self.buttonMap = {}
end

function GPM:exists(joyId)
   return self.isConnected[joyId] == nil and self.isConnected[joyId]
end

function GPM:getStick(joyId)
   return self.connectedSticks[joyId]
end


-- Returns true if given button was just pressed THIS frame!
function GPM:buttonDown(joyId, button)
 --  print(self.isConnected[joyId])
   if self.isConnected[joyId] == nil or self.isConnected[joyId] == false then return false
   else
      return self.buttonMap[joyId][button] == true
   end
end

-- Returns true if given button was just released THIS frame!
function GPM:buttonUp(joyId, button)
   if self.isConnected[joyId] == nil or self.isConnected[joyId] == false then return false
   else
      return self.buttonMap[joyId][button] == false
   end
end

-- returns the instantenous state of the requested button for the given joystick 
function GPM:button(joyId, button)
   local stick = self.connectedSticks[joyId]
   if self.isConnected[joyId] == nil or self.isConnected[joyId] == false then return false end

   local isDown = stick:isGamepadDown(button)

   -- Do we want to convert the left analog stick to dpad buttons?
   if self.adEnabled and not isDown then
      local xAxis = stick:getGamepadAxis("leftx")
      local yAxis = stick:getGamepadAxis("lefty")
      if button == 'dpright' then
	 isDown = xAxis > DEAD_ZONE
      elseif button == 'dpleft' then
	 isDown = xAxis < -DEAD_ZONE
      elseif button == 'dpup' then
	 isDown = yAxis < -DEAD_ZONE
      elseif button == 'dpdown' then
      isDown = yAxis > DEAD_ZONE
      end
   end

   return isDown
end

function GPM:update(dt)
   --   for i = 1, #self.connectedSticks do
   for i = 1, #self.isConnected do
      if self.buttonMap[i] then
	 for k,_ in pairs(self.buttonMap[i]) do
	    self.buttonMap[i][k] = nil
	 end
      end
   end
end


return GPM

-- --------------------------------------------------------------------------------

-- local Class = require("lib.Class")

-- local GPM = Class:derive("GamepadMgr")

-- local DEAD_ZONE = 0.1

-- local function hookLoveEvents(self)
   
--    function love.joystickadded(joystick)
--       local id = joystick:getID()
--       assert(self.connectedSticks[id] == nil, "HATA: joystick " .. id .. " already exists!")
--       self.connectedSticks[id] = joystick
--       self.isConnected[id] = true
--       print("connected: " .. id)
--       self.buttonMap[id] = {}
--       -- TODO: invoke a 'controllerAdded' event here.
      
--    end

--    function love.joystickremoved(joystick)
--       local id = joystick:getID()
--       self.isConnected[id] = false
--       self.connectedSticks[id] = nil
--       self.buttonMap[id] = {}
--       -- TODO: invoke a 'controllerRemoved' event here.
--    end

--    function love.gamepadpressed(joystick, button)
--       local id = joystick:getID()
--       self.buttonMap[id][button] = true
--    end

--    function love.gamepadreleased(joystick, button)
--       local id = joystick:getID()
--       self.buttonMap[id][button] = false
--    end
   
-- end


-- function GPM:new(dbFiles, adEnabled)
--    if dbFiles ~= nil then
--       for i = 1, #dbFiles do
-- 	 love.joystick.loadGamepadMappings(dbFiles[i])
--       end
--    end

--    -- If it is true then the left analog joystick will be converted
--    -- it's corresponding dpad button output
--    self.adEnabled = adEnabled
   
--    -- The currently-connected joysticks
--    self.connectedSticks = {}
--    self.isConnected = {}

--    -- Maps a joystick id to a table of key values
--    -- where key is a button and the value is either true = justPressed
--    -- false = justReleased, or nil = none
--    self.buttonMap = {}
   
--    hookLoveEvents(self)
-- end

-- function GPM:exists(joyId)
--    return self.isConnected[joyId] == nil and self.isConnected[joyId]
-- end


-- -- Returns true if the given button was just pressed THIS frame!
-- function GPM:buttonDown(joyId, button) 
--    if self.isConnected[joyId] == nil or self.isConnected[joyId] == false  then return false 
--    else return self.buttonMap[joyId][button] == true
--    end
-- end

-- -- Returns true if the given button was just released THIS frame!
-- function GPM:buttonUp(joyId, button)
--    if self.isConnected[joyId] == nil or self.isConnected[joyId] == false  then return false
--    else
--       return self.buttonMap[joyId][button] == false
--    end
-- end

-- -- return the instantenous state of the requested button for the given
-- --joystick
-- function GPM:button(joyId, button)
--    local stick = self.connectedSticks[joyId]
--    if self.isConnected[joyId] == nil or self.isConnected[joyId] == false  then return false end

   
--    local isDown = stick:isGamepadDown(button)

--    -- Do we want to convert the left analog stick to the dpad buttons?
--    if self.adEnabled and not isDown then
--       local xAxis = stick:getGamepadAxis("leftx")
--       local yAxis = stick:getGamepadAxis("lefty")
--       if button == 'dpright' then
-- 	 isDown = xAxis > DEAD_ZONE
--       elseif button == 'dpleft' then 
-- 	 isDown = xAxis < -DEAD_ZONE
--       elseif button == 'dpup' then
-- 	 isDown = yAxis < -DEAD_ZONE
--       elseif button == 'dpdown' then
-- 	 isDown = yAxis > -DEAD_ZONE
--       end
--    end


--    return isDown
-- end

-- function GPM:update(dt)
--    for i = 1, #self.connectedSticks do
--       if self.buttonMap[i] then
-- 	 for k,_ in pairs(self.buttonMap[i]) do
-- 	    self.buttonMap[i][k]  = nil
-- 	 end
--       end
--    end
-- end


-- return GPM







